/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.exception;

import ws.ladder.util.I18NUtils;

public class PropertyNotFoundException extends ApplicationException {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	
	private String objectTypeName;
	private String objectId;
	
	public PropertyNotFoundException() {
		super();
	}
	
	public PropertyNotFoundException(String objectTypeName, String objectId, Exception e) {
		super(e);
		this.objectTypeName = objectTypeName;
		this.objectId = objectId;		
	}
	
	public PropertyNotFoundException(String objectTypeName, String objectId) {
		this.objectTypeName = objectTypeName;
		this.objectId = objectId;		
	}
	
	public PropertyNotFoundException(Throwable t) {
		super(t);
	}
	
	@Override
	public String getMessage() {
		String[] params = new String[2];
		params[0] = objectTypeName;
		params[1] = objectId;
		return  I18NUtils.getResource(  "exception.objectNotFound",I18NUtils.CORE_RESOURCES, params);		
	}

	public String getObjectTypeName() {
		return objectTypeName;
	}

	public void setObjectTypeName(String objectTypeName) {
		this.objectTypeName = objectTypeName;
	}

	public String getObjectId() {
		return objectId;
	}

	public void setObjectId(String objectId) {
		this.objectId = objectId;
	}
}
