/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model;

import java.io.Serializable;

/**
 * Uniquely identifies a persistent entity.
 */
public final class EntityKey implements Serializable {
	private static final long serialVersionUID = 1L;
	
	private final Object identifier;
	private final String referenceTypeName;
	private final String derivedTypeName;
	private final int hashCode;

	public EntityKey(Object id, String referenceTypeName, String derivedTypeName) {
		if ( id == null ) {
			throw new IllegalStateException( "null identifier" );
		}
		this.identifier = id; 
		this.referenceTypeName = referenceTypeName;
		this.derivedTypeName = derivedTypeName;
		this.hashCode = generateHashCode();
	}

	private int generateHashCode() {
		int result = 17;
		result = 37 * result + referenceTypeName.hashCode();
		result = 37 * result + derivedTypeName.hashCode();
		result = 37 * result + identifier.hashCode();
		return result;
	}

	public Object getIdentifier() {
		return identifier;
	}

	@Override
	public boolean equals(Object other) {
		EntityKey otherKey = (EntityKey) other;
		return otherKey.referenceTypeName.equals(this.referenceTypeName) &&
				otherKey.derivedTypeName.equals(this.derivedTypeName) &&
				otherKey.identifier.equals(this.identifier);
	}

	@Override
	public int hashCode() {
		return hashCode;
	}

}
