/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.view;

import java.util.HashMap;
import java.util.Map;

import org.hibernate.SessionFactory;
import org.springframework.beans.factory.annotation.Autowired;

import ws.ladder.model.CallInfo;
import ws.ladder.util.HibernateUtil;

public class HibernateQueryBuilder extends QueryBuilder {

	@Autowired
	protected SessionFactory sessionFactory;

	@Override
	public Query createQuery(CallInfo callInfo) {
		Map<String, Object> mutableFilters = new HashMap<String, Object>(callInfo.getSettings().getFilters());
		String queryString = constructQuery(callInfo, mutableFilters);
		
		Query result = null;
		if(isSQL()) {
			result = new HibernateQuery(HibernateUtil.getCurrentSession(sessionFactory).createSQLQuery(queryString));
		} else
			result = new HibernateQuery(HibernateUtil.getCurrentSession(sessionFactory).createQuery(queryString));

		postProcess(callInfo, result, mutableFilters);

		return result;
	}

}
