/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.model.view;

import java.util.HashSet;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

public class JPAQuery implements Query {
	
	private javax.persistence.Query jpaQuery;

	public JPAQuery(javax.persistence.Query jpaQuery) {
		this.jpaQuery = jpaQuery;
	}

	@SuppressWarnings("rawtypes")
	@Override
	public List getResultList(ViewBranch viewBranch) {
		return jpaQuery.getResultList();
	}

	@Override
	public Object getSingleResult() {
		return jpaQuery.getSingleResult();
	}

	@Override
	public void setParameter(String name, Object value) {
		jpaQuery.setParameter(name, value);
	}

	@Override
	public boolean hasParameter(String name) {
		Set<String> paramNames = new HashSet<String>();
		
		Iterator<javax.persistence.Parameter<?>> iter = jpaQuery.getParameters().iterator();
		while(iter.hasNext())
			paramNames.add(iter.next().getName());
		
		return paramNames.contains(name);
	}

	@Override
	public void setMaxResults(int limit) {
		jpaQuery.setMaxResults(limit);
	}

	@Override
	public void setFirstResult(int offset) {
		jpaQuery.setFirstResult(offset);
	}	
}
