/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.service;

import java.util.Map;

import ws.ladder.model.Type;

public interface PersistenceOrchestrator {
	
	/**
	 * Find the entity by its primary key
	 * @param id
	 * @return
	 */
	public Object findById(Class<?> persistentClass, Object id);	
	
	/**
	 * Find the entity by one or more property values
	 * @param id
	 * @return
	 */
	public Object findByProperty(Type type, Map<String, String> propertyValues);		

    /**
     * Save the entity in the persistence store
     * @param entity
     * @return
     */
    public void saveOrUpdate(Object entity);
    
    /**
     * Delete the entity from the persistence store
     * @param entity
     */
    public void delete(Object entity);
    
    /**
     * Flush any buffered content to the persistence store
     */
    public void flush();
    
    /**
     * Indicates if this persistence orchestrator automatically tracks version 
     * If not, the framework takes care of updating the version in order to support optimistic concurrency control
     */
    public boolean supportsVersionTracking();
    
    /**
     * Returns the mechanisms supported for querying
     * @return
     */
    public QueryCapability getQueryCapability();
    
    /**
     * Some ORMs cannot resolve all the issues while persisting an aggregate. 
     * For e.g., cycle dependencies within an aggregate is a problem while persisting it.
     * 
     * This is usually not a problem with NoSQL databases.
     * Setting this flag to false, allows the framework to individually persist
     * objects within the aggregate in based on topological ordering of the required fields.
     */
    public boolean canProcessAggregate();
}