/**
 * Ladder, empowering Model Driven Architecture in J2EE applications
 *
 * Copyright (c) 2012, Dilip Dalton or third-party contributors as
 * indicated by the @author tags.
 *
 * This copyrighted material is made available to anyone wishing to use, modify,
 * copy, or redistribute it subject to the terms and conditions of the GNU
 * Lesser General Public License, as published by the Free Software Foundation.
 *
 * Ladder is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this distribution. If not, see <http://www.gnu.org/licenses/>.
 */

package ws.ladder.util;

import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.cfg.Configuration;
import org.hibernate.persister.collection.CollectionPersister;
import org.hibernate.type.CollectionType;
import org.hibernate.type.Type;

public class HibernateUtil {
	public static CollectionPersister getCollectionPersister(SessionFactory sessionFactory, CollectionType collType) {

		try {
			return (new Hibernate4Support()).getCollectionPersister(sessionFactory, collType);
		} catch(NoClassDefFoundError e) {
			return getCollectionPersisterHibernate3(sessionFactory, collType);
		} catch(NoSuchMethodError nme) {
			return getCollectionPersisterHibernate3(sessionFactory, collType);
		}
	}
	
	private static CollectionPersister getCollectionPersisterHibernate3(SessionFactory sessionFactory, CollectionType collType) {
		return (new Hibernate3Support()).getCollectionPersister(sessionFactory, collType);
	}

	public static boolean isCascaded(Configuration configuration, String entityName, String inputPropertyName) {	
		try {
			return (new Hibernate4Support()).isCascaded(configuration, entityName, inputPropertyName);
		} catch(NoClassDefFoundError e) {
			return isCascadedHibernate3(configuration, entityName, inputPropertyName);
		} catch(NoSuchMethodError nme) {
			return isCascadedHibernate3(configuration, entityName, inputPropertyName);
		}
	}

	private static boolean isCascadedHibernate3(Configuration configuration, String entityName, String inputPropertyName) {
		return (new Hibernate3Support()).isCascaded(configuration, entityName, inputPropertyName);
	}

	public static Type getEntityType(SessionFactory sessionFactory, String entityName) {
		try {
			return (new Hibernate4Support()).getEntityType(sessionFactory, entityName);
		} catch(NoClassDefFoundError e) {
			return getEntityTypeHibernate3(sessionFactory, entityName);
		} catch(NoSuchMethodError nme) {
			return getEntityTypeHibernate3(sessionFactory, entityName);			
		}
	}
	
	private static Type getEntityTypeHibernate3(SessionFactory sessionFactory, String entityName) {
		return (new Hibernate3Support()).getEntityType(sessionFactory, entityName);
	}
	
	public static Session getCurrentSession(SessionFactory sessionFactory) {
		try {
			return (new Hibernate4Support()).getCurrentSession(sessionFactory);
		} catch(NoClassDefFoundError e) {
			return getCurrentSessionHibernate3(sessionFactory);
		} catch(NoSuchMethodError nme) {
			return getCurrentSessionHibernate3(sessionFactory);			
		}
	}
	
	private static Session getCurrentSessionHibernate3(SessionFactory sessionFactory) {
		return (new Hibernate3Support()).getCurrentSession(sessionFactory);
	}	
}
